# This file contains data for importing recipes from common recipe
# websites. If the HTML is consistent, we can write add import support
# simply by adding another set of rules to this file and registering
# it in the dictionary at the bottom of the file.
#
# Editing this file will require consistent python syntax, but
# shouldn't require much if any programming knowledge beyond knowledge
# of HTML and regular expressions. Hopefully advanced users can
# contribute new import sites relatively easily.
#
# To support a new import, you need to create a new set of rules and
# then "register" it by adding it to the dictionaries SUPPORTED_URLS
# (the canonical name -- e.g. www.epicurious.com) and
# SUPPORTED_URLS_REGEXPS (a regexp to match any funny variants of that
# name -- .*epicurious.com.*)
#
# BASIC_TEMPLATE
#
# mywebsite_rules = [
#     ['attribute',
#      [PATH_TO_TAG],
#      'text', 'markup' OR 'attname'
#      post_processing # Either a function that returns a string or a dictionary
#                      # Or a tuple (regexp, force_match)
#                      # if force_match, then return "" if there is no match
#                      # else, return the unmodified string if there is no match
#                      # If a function, return a string for most cases. Return a dictionary
#                      # if we're doing ingredient parsing ({'amt':1,'unit':'cup',...})
#                      #
#                      # If a function, we take two args -- value (string or markup) and tag (tag object)
#                      # e.g.
#                      # def post_proc (value, tag): ... return "something"
#
#      ],
#     ...
#    ]
#
# attribute can be: title, servings, image, source, preptime,
# cooktime, instructions, modifications, ingredient_block (string) or ingredient_parsed (dictionary)
#
# attributes can be specified more than once, in which case there will
# be added to. For example, you might want to add various parts of the
# webpage to your instructions.
#
# PATH_TO_TAG is a list of search criteria that zero in on the HTML
# element we want to find, drilling down as necessary (see below)
#
# 'text' or 'markup' specifies whether we want markup when we grab the value
#
# POST_PROCESSING can be a function or a regexp.  If it is a regexp,
# it should have a grouping construct which will contain the text we
# want to keep.
#
# Each instruction in the PATH_TO_TAG is a python dictionary
# containing the following entries
#
# 'regexp': REGEXP a regular expression text search; if this is
#                  specified, anything else is ignored
# 'string': STRING a string text search; if this is specified, 
#                  everything else is ignored
# OR...
#
# 'tag': TAGNAME                  HTML Element Name
# 'attributes':{attname:value}    Attributes on the Element we're looking for
# 'index': INTEGER or [START,END] python index for the count of the tag
#                                 we start counting from 0
#
# EXAMPLES OF RULES
# 
# Here are some examples of how a rule can look.
#
# EXAMPLE 1
#
# Get the 'image' attribute from the IMG tag in the 2nd row of the
# first <table class='recipe'>. (Note: for img, we want the HTML img
# tag with its markup; our importer class will do the right thing and
# grab the image from the src attribute)
#
# <table class='recipe'><tr>...</tr><tr>...<IMG></tr></table>
# 
# ['image',
#  [{'tag':'table',{'class':'recipe'},'index':0},
#   {'tag':'tr','index':1},
#   {'tag':'img','index':0}],
#  'markup']
#
# EXAMPLE 2
#
# Get our "servings" value by searching all the text for the phrase
# "X servings"
#
# ['servings',
#  [{'regexp':'[0-9]+\s+servings?'}],
#  'text',
#  '([0-9]+)\s+servings?'] # our grouping construct will grab just the number.
#
# EXAMPLE 3
#
# Get our instructions from all <p class='body'> elements.
#
# ['instructions',
# [{'tag':'p',
#   'attributes':{'class':'body'},
#   'index':[0,None]
#   }],
# 'text']
#
# For a further sense of how things work, take a look at the actual
# rulesets below.
#
# You can test out rules as you write them by running this file as a
# script directly at a console.

import re
import html_importer
from gourmet import convert

epicurious_rules = [
    ['title',
     [{'tag':'div',
       'attributes':{'class':'recipehed'},}],
     'text',
     lambda s,t: s.capitalize()],
    ['image',
     [{'tag':'img',
       'attributes':{'class':'recipeimg'},}],
     'src',
     ],
    ['ingredient_block',
     [{'tag':'div',
       'attributes':{'class':'copy'},
       'index':1,
       }],
     'text'],
    ['source',
     [{'tag':'div',
       'attributes':{'class':'magname'},
       }],
     'text'],
    ['source',
     [{'tag':'div',
       'attributes':{'class':'date'},
       }],
     'text'],
    ['servings',
     [{'regexp':'Makes\s+%(num)s+\s+servings?'%{'num':convert.NUMBER_REGEXP}}],
     'text',
     ('Makes\s+(%(num)s+)\s+servings?'%{'num':convert.NUMBER_REGEXP},True)],
    ['servings',
     [{'regexp':'Serves\s+%(num)s+'%{'num':convert.NUMBER_REGEXP}}],
     'text',
     ('Serves\s+(%(num)s+)'%{'num':convert.NUMBER_REGEXP},True)],
    ['instructions',
     [{'tag':'div',
       'attributes':{'class':'copy'},
       'index':2,}],
     'text'],
    
    ]

ing_match = re.compile('(^|\n)\s*%(num)s+[^.].*'%{'num':convert.NUMBER_REGEXP})

def keep_ing (txt,tag):
    if ing_match.search(txt):
        return txt

def reject_ing (txt,tag):
    if not ing_match.search(txt):
        return txt

eating_well_mainrec = {'tag':'div',
                       'attributes':{'align':'center'},
                       'index':1}
eating_well_rules = [
    ['image',
     [{'tag':'img','index':2}],
     'src',
     ],
    ['title',
     [{'tag':'title'}],
     'text',
     ('EatingWell (.*)',False)
     ],
    ['source',
     [{'tag':'font',
       'attributes':{'face':'Arial, Helvetica, sans-serif','color':'#000000'},}],
     'text',
     lambda txt,tag: 'Eating Well %s'%txt
     ],
    ['instructions',
     [eating_well_mainrec,
      {'tag':'p',
       'attributes':{'align':'left'},
       'index':[0,None]},
      ],
     'text',
     reject_ing,
     ],
    ['ingredient_block',
     [eating_well_mainrec,
      {'tag':'p',
       'attributes':{'align':'left'},
       'index':[0,None]},
      ],
     'text',
     keep_ing,
     ],
    ['servings',
     [eating_well_mainrec,
      {'regexp':'%(num)s+ [Ss]ervings?'%{'num':convert.NUMBER_REGEXP}},
      ],
     'text',
     ('(%(num)s+) [Ss]ervings?'%{'num':convert.NUMBER_REGEXP},True)],
    ['preptime',
     [eating_well_mainrec,
      {'regexp':'(Prep(aration)|Active)?\s*[tT]ime.*minutes','index':0}],
     'text',
     ('(%(num)s+ (minutes|hours|days))'%{'num':convert.NUMBER_REGEXP},True),
     ],
    ['cooktime',
     [eating_well_mainrec,
      {'regexp':'(Start\s*to\s*finish|Total).*minutes','index':0}],
     'text',
     ('(%(num)s+ (minutes|hours|days))'%{'num':convert.NUMBER_REGEXP},True)
     ],
    # newer format preptime and cooktime...
    ['preptime',
     [{'tag':'i'},
      {'regexp':'.*minutes','index':0}],
     'text',
     ('\s*(%(num)s+ (minutes|hours|days))'%{'num':convert.NUMBER_REGEXP},True)
     ],
    ['cooktime',
      [{'tag':'i'},
       {'regexp':'.*minutes','index':1}],
     'text',
     ('\s*(%(num)s+ (minutes|hours|days))'%{'num':convert.NUMBER_REGEXP},True)
     ],
    ]

all_recipes_rules = [
    ['title',
     [{'tag':'h2','index':0}],
     'text'],
    ['source',
     [{'tag':'div',
      'attributes':{'class':'V10color'},}],
     'text',
     ('Submitted\s+by:\s+(.*)',True)
     ],
    ['ingredient_block',
     [{'tag':'ul',
       'attributes':{'class':'arlist','style':'list-style: none;'}
       },
      {'tag':'li',
       'index':[0,None]}
      ],
     'text'],
    ['instructions',
     [{'tag':'meta',
       'attributes':{'name':'description'},
       }],
     'content',
     ],
    ['instructions',
     [{'tag':'ol',
       'attributes':{'class':'arlist'},
       },
      {'tag':'li',
       'index':[0,None],
       },
      ],
     'text',],
    ['preptime',
     [{'tag':'dd',
       'index':0}],
     'text'],
    ['cooktime',
     [{'tag':'dd',
       'index':1}],
     'text'],
    ['servings',
     [{'tag':'dd',
       'index':3}],
     'text',
     '(%(num)s+).*'%{'num':convert.NUMBER_REGEXP}],
    ]
recipezaar_group = None

def recipezaar_ingredient_parser (text, tag):
    """Do the right thing when handed something like... 
    <td class="amt" align="right" nowrap="nowrap" valign="top">
    1
    </td>
    <td valign="top">
    dozen
    medium sized <b>prawns</b>, peeled and deveined		 	 	 	 			 </td>
    </tr>"""
    global recipezaar_group
    ret = {}
    if recipezaar_group: ret['inggroup']=recipezaar_group
    # groups are in B in DIV
    grouptag = tag.fetch('div')
    if grouptag: grouptag = grouptag[0].fetch('b')
    if grouptag:
        recipezaar_group = html_importer.get_text(grouptag[0])
        return
    keytag = tag.fetch('b')
    if keytag:
        keytag=keytag[0]
        ret['ingkey']=html_importer.get_text(keytag)
    amttag = tag.fetch('td','amt')
    if amttag:
        amttag = amttag[0]
        ret['amount']=html_importer.get_text(amttag)
    txt=html_importer.get_text(tag.fetch('td')[-1])
    words = re.split('\s+',txt)
    if len(words)>1 and words[0] not in ret['ingkey']:
        ret['unit']=words[0]
        ret['item']=" ".join(words[1:])
    else:
        ret['item']=" ".join(words)
    return ret
    
recipezaar_rules = [
    ['title',
     [{'tag':'title'}],
     'text',
     ],
    ['image',
     [{'tag':'img',
       'attributes':{'alt':'Recipe Photo'}}],
     'src',
     ],
    ['instructions',
     [{'tag':'div',
       'attributes':{'class':'recipetext'},
       }],
     'text'],
    ['ingredient_parsed',
     [{'tag':'table',
       'attributes':{'class':'ingred'}},
      {'tag':'tr',
       'index':[0,None]}],
     'text',
     recipezaar_ingredient_parser,
      ],
    ['servings',
     [{'regexp':'%(num)s+\s+servings?'%{'num':convert.NUMBER_REGEXP}}],
     'text',
     ('(%(num)s+).+'%{'num':convert.NUMBER_REGEXP},True)],
    ['preptime',
     [{'regexp':'\s*(%(num)s+\s*(minutes?|hours?|days?|weeks?|months?))'%{'num':convert.NUMBER_REGEXP}}],
     'text',
     ('\s*(%(num)s+\s*(minutes?|hours?|days?|weeks?|months?))'%{'num':convert.NUMBER_REGEXP},False)
     #[{'tag':'h4',
     #  'attributes':{'style':'font-weight: normal;'},
     #  'index':[0,None]},],
     #'text',
     #('\s*(%(num)s+\s*(minutes?|hours?|days?|weeks?|months?))'%{'num':convert.NUMBER_REGEXP},True)
    ],
    ['instructions',
     [{'tag':'span',
       'attributes':{'class':'recipetext'},
       'index':[0,None]
       }],
     'text',
     ],
    ]
      




recipebook_rules = [
    ['title',
     [{'tag':'font',
       'attributes':{'size':'4'}
       }],
     'text',
     ],
    ['ingredient_block',
     [{'tag':'td',
       'attributes':{'width':'100%',
                     'align':'center'},
       'index':[0,None],
       }],
     'text',
     ],
    ['instructions',
     [{'tag':'td',
       'attributes':{'width':'396'},
       'index':4,
       }],
     'text',
     ('Directions:\s+((.*\n)+)',False) # chop off "Directions: "
     ],
    ['instructions',
     [{'tag':'td',
       'attributes':{'width':'396'},
       'index':6,
       }],
     'text',
    ],
    ['source',
     [{'tag':'td',
       'attributes':{'width':'396'},
       'index':6,
       }],
     'text',
     ('Submitted\s+By:\s+(.*)\n',True),
     ],
    ['preptime',
     [{'tag':'font',
       'attributes':{'size':'2'},}],
     'text',
     ('Preparation Time: (.*)',True)
     ],
    ['cooktime',
     [{'tag':'font',
       'attributes':{'size':'2'},}],
     'text',
     ('Cooking Time: (.*)',True)
     ],
    ['servings',
     [{'tag':'font',
       'attributes':{'size':'2'},}],
     'text',
     ('Number\s+of\s+Servings\s*:\s*(%(num)s+).*'%{'num':convert.NUMBER_REGEXP},True)
     ] ,   
    ]

SUPPORTED_URLS = {
    'www.allrecipes.com':all_recipes_rules,
    'www.eatingwell.com':eating_well_rules,
    'www.epicurious.com':epicurious_rules,
    'www.recipebookonline.com':recipebook_rules,
    'www.recipezaar.com':recipezaar_rules,
                  }

SUPPORTED_URLS_REGEXPS = [('.*eatingwell\.com',eating_well_rules),
                          ('.*\.allrecipes\.com',all_recipes_rules),
                          ]

if __name__ == '__main__':
    print 'Testing Interface for Gourmet HTML import.'
    print '------------------------------------------'
    print 'Import supported for: '
    for s in SUPPORTED_URLS.keys():
        print '               ',s
    print '\n'
    url = raw_input('Enter URL to import: ')
    import html_importer
    print 'Importing URL: ',url
    d=html_importer.scrape_url(url)
    print '---------------'
    for k,v in d.items():
        if k=='image' and v:
            display_image = raw_input('Display image? (yes or no):').lower()
            if display_image=='y' or display_image=='yes':
                ofi = open('/tmp/gourmet_image_test','w')
                ofi.write(html_importer.get_image_from_tag(v,url))
                ofi.close()
                import os
                os.popen('display /tmp/gourmet_image_test','w')
        else:
            if type(v)==list:
                print k,':'
                for i in v: print i                
            else:
                print k,' :\t',v
            
